Module: RingSig

Defined in:
lib/ring_sig.rb,
lib/ring_sig/hasher.rb,
lib/ring_sig/version.rb,
lib/ring_sig/signature.rb,
lib/ring_sig/public_key.rb,
lib/ring_sig/private_key.rb

Overview

The top-level module for the RingSig gem.

Defined Under Namespace

Classes: Hasher, PrivateKey, PublicKey, Signature

Constant Summary collapse

VERSION =

This gem's version.

'0.2.0'

Class Attribute Summary collapse

Class Attribute Details

.default_groupECDSA::Group

Returns the default group. This group will be used in any method in the RingSig library that calls for a group, if none is specified. Starts as ECDSA::Group::Secp256k1.

Returns:

  • (ECDSA::Group)

    the default group. This group will be used in any method in the RingSig library that calls for a group, if none is specified. Starts as ECDSA::Group::Secp256k1.



16
17
18
# File 'lib/ring_sig.rb', line 16

def default_group
  @default_group
end

.default_hash_algorithm#digest

Returns the default hash algorithm. This hash algorithm will be used in any method in the RingSig library that calls for a hash algorithm, if none is specified. Starts as OpenSSL::Digest::SHA256.

Returns:

  • (#digest)

    the default hash algorithm. This hash algorithm will be used in any method in the RingSig library that calls for a hash algorithm, if none is specified. Starts as OpenSSL::Digest::SHA256.



21
22
23
# File 'lib/ring_sig.rb', line 21

def default_hash_algorithm
  @default_hash_algorithm
end